@@ -3,48 +3,20 @@
 #
 # Copyright 2013 Red Hat, Inc.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
-import logging
 import os
 import re
 import stat
+import subprocess
 
 import libvirt
 
+from .logger import log
 from .storage import StoragePool, StorageVolume
 
 
-def _lookup_pool_by_dirname(conn, path):
-    """
-    Try to find the parent pool for the passed path.
-    If found, and the pool isn't running, attempt to start it up.
-
-    return pool, or None if not found
-    """
-    pool = StoragePool.lookup_pool_by_path(conn, os.path.dirname(path))
-    if not pool:
-        return None
-
-    # Ensure pool is running
-    if pool.info()[0] != libvirt.VIR_STORAGE_POOL_RUNNING:
-        pool.create(0)
-    return pool
-
-
 def _lookup_vol_by_path(conn, path):
     """
     Try to find a volume matching the full passed path. Call info() on
@@ -97,7 +69,7 @@ def _stat_disk(path):
     return True, 0
 
 
-def check_if_path_managed(conn, path):
+def _check_if_path_managed(conn, path):
     """
     Try to lookup storage objects for the passed path.
 
@@ -107,15 +79,15 @@ def check_if_path_managed(conn, path):
     if vol:
         return vol, vol.storagePoolLookupByVolume()
 
-    pool = _lookup_pool_by_dirname(conn, path)
+    pool = StoragePool.lookup_pool_by_path(conn, os.path.dirname(path))
     if not pool:
         return None, None
 
     # We have the parent pool, but didn't find a volume on first lookup
-    # attempt. Refresh the pool and try again, incase we were just out
-    # of date.
+    # attempt. Refresh the pool and try again, in case we were just out
+    # of date or the pool was inactive.
     try:
-        pool.refresh(0)
+        StoragePool.ensure_pool_is_running(pool, refresh=True)
         vol, verr = _lookup_vol_by_path(conn, path)
         if verr:
             try:
@@ -151,14 +123,14 @@ def manage_path(conn, path):
     """
     If path is not managed, try to create a storage pool to probe the path
     """
-    if not conn.check_support(conn.SUPPORT_CONN_STORAGE):
+    if not conn.support.conn_storage():
         return None, None
     if not path:
         return None, None
 
-    if not path_is_url(path):
+    if not path_is_url(path) and not path_is_network_vol(conn, path):
         path = os.path.abspath(path)
-    vol, pool = check_if_path_managed(conn, path)
+    vol, pool = _check_if_path_managed(conn, path)
     if vol or pool or not _can_auto_manage(path):
         return vol, pool
 
@@ -167,7 +139,7 @@ def manage_path(conn, path):
     if not poolname:
         poolname = "dirpool"
     poolname = StoragePool.find_free_name(conn, poolname)
-    logging.debug("Attempting to build pool=%s target=%s", poolname, dirname)
+    log.debug("Attempting to build pool=%s target=%s", poolname, dirname)
 
     poolxml = StoragePool(conn)
     poolxml.name = poolname
@@ -185,7 +157,20 @@ def path_is_url(path):
     """
     if not path:
         return False
-    return bool(re.match("[a-zA-Z]+(\+[a-zA-Z]+)?://.*", path))
+    return bool(re.match(r"[a-zA-Z]+(\+[a-zA-Z]+)?://.*", path))
+
+
+def path_is_network_vol(conn, path):
+    """
+    Detect if path is a network volume such as rbd, gluster, etc
+    """
+    if not path:
+        return False
+
+    for volxml in conn.fetch_all_vols():
+        if volxml.target_path == path:
+            return volxml.type == "network"
+    return False
 
 
 def _get_dev_type(path, vol_xml, vol_object, pool_xml, remote):
@@ -234,13 +219,150 @@ def _get_dev_type(path, vol_xml, vol_object, pool_xml, remote):
     return "file"
 
 
+def path_definitely_exists(conn, path):
+    """
+    Return True if the path certainly exists, False if we are unsure.
+    See DeviceDisk entry point for more details
+    """
+    if path is None:
+        return False
+
+    try:
+        (vol, pool) = _check_if_path_managed(conn, path)
+        ignore = pool
+        if vol:
+            return True
+
+        if not conn.is_remote():
+            return os.path.exists(path)
+    except Exception:
+        pass
+
+    return False
+
+
+#########################
+# ACL/path perm helpers #
+#########################
+
+def _fix_perms_acl(dirname, username):
+    cmd = ["setfacl", "--modify", "user:%s:x" % username, dirname]
+    proc = subprocess.Popen(cmd,
+                            stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
+    out, err = proc.communicate()
+
+    log.debug("Ran command '%s'", cmd)
+    if out or err:
+        log.debug("out=%s\nerr=%s", out, err)
+
+    if proc.returncode != 0:
+        raise ValueError(err)
+
+
+def _fix_perms_chmod(dirname):
+    log.debug("Setting +x on %s", dirname)
+    mode = os.stat(dirname).st_mode
+    newmode = mode | stat.S_IXOTH
+    os.chmod(dirname, newmode)
+    if os.stat(dirname).st_mode != newmode:
+        # Trying to change perms on vfat at least doesn't work
+        # but also doesn't seem to error. Try and detect that
+        raise ValueError(_("Permissions on '%s' did not stick") %
+                         dirname)
+
+
+def set_dirs_searchable(dirlist, username):
+    useacl = True
+    errdict = {}
+    for dirname in dirlist:
+        if useacl:
+            try:
+                _fix_perms_acl(dirname, username)
+                continue
+            except Exception as e:
+                log.debug("setfacl failed: %s", e)
+                log.debug("trying chmod")
+                useacl = False
+
+        try:
+            # If we reach here, ACL setting failed, try chmod
+            _fix_perms_chmod(dirname)
+        except Exception as e:
+            errdict[dirname] = str(e)
+
+    return errdict
+
+
+def _is_dir_searchable(dirname, uid, username):
+    """
+    Check if passed directory is searchable by uid
+    """
+    if "VIRTINST_TEST_SUITE" in os.environ:
+        return True
+
+    try:
+        statinfo = os.stat(dirname)
+    except OSError:
+        return False
+
+    if uid == statinfo.st_uid:
+        flag = stat.S_IXUSR
+    elif uid == statinfo.st_gid:
+        flag = stat.S_IXGRP
+    else:
+        flag = stat.S_IXOTH
+
+    if bool(statinfo.st_mode & flag):
+        return True
+
+    # Check POSIX ACL (since that is what we use to 'fix' access)
+    cmd = ["getfacl", dirname]
+    try:
+        proc = subprocess.Popen(cmd,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE)
+        out, err = proc.communicate()
+    except OSError:
+        log.debug("Didn't find the getfacl command.")
+        return False
+
+    if proc.returncode != 0:
+        log.debug("Cmd '%s' failed: %s", cmd, err)
+        return False
+
+    pattern = "user:%s:..x" % username
+    return bool(re.search(pattern.encode("utf-8", "replace"), out))
+
+
+def is_path_searchable(path, uid, username):
+    """
+    Check each dir component of the passed path, see if they are
+    searchable by the uid/username, and return a list of paths
+    which aren't searchable
+    """
+    if os.path.isdir(path):
+        dirname = path
+        base = "-"
+    else:
+        dirname, base = os.path.split(path)
+
+    fixlist = []
+    while base:
+        if not _is_dir_searchable(dirname, uid, username):
+            fixlist.append(dirname)
+        dirname, base = os.path.split(dirname)
+
+    return fixlist
+
+
 ##############################################
 # Classes for tracking storage media details #
 ##############################################
 
 class _StorageBase(object):
     """
-    Storage base class, defining the API used by VirtualDisk
+    Storage base class, defining the API used by DeviceDisk
     """
     def __init__(self, conn):
         self._conn = conn
@@ -353,7 +475,7 @@ class _StorageCreator(_StorageBase):
         if err:
             raise ValueError(msg)
         if msg:
-            logging.warning(msg)
+            log.warning(msg)
 
     def will_create_storage(self):
         return True
@@ -391,7 +513,7 @@ class CloneStorageCreator(_StorageCreator):
         else:
             vfs = os.statvfs(os.path.dirname(self._path))
             avail = vfs.f_frsize * vfs.f_bavail
-        need = long(self._size * 1024 * 1024 * 1024)
+        need = int(self._size) * 1024 * 1024 * 1024
         if need > avail:
             if self._sparse:
                 msg = _("The filesystem will not have enough free space"
@@ -411,7 +533,7 @@ class CloneStorageCreator(_StorageCreator):
         text = (_("Cloning %(srcfile)s") %
                 {'srcfile': os.path.basename(self._input_path)})
 
-        size_bytes = long(self.get_size() * 1024 * 1024 * 1024)
+        size_bytes = int(self.get_size()) * 1024 * 1024 * 1024
         progresscb.start(filename=self._output_path, size=size_bytes,
                          text=text)
 
@@ -422,10 +544,10 @@ class CloneStorageCreator(_StorageCreator):
         if self._input_path == "/dev/null":
             # Not really sure why this check is here,
             # but keeping for compat
-            logging.debug("Source dev was /dev/null. Skipping")
+            log.debug("Source dev was /dev/null. Skipping")
             return
         if self._input_path == self._output_path:
-            logging.debug("Source and destination are the same. Skipping.")
+            log.debug("Source and destination are the same. Skipping.")
             return
 
         # If a destination file exists and sparse flag is True,
@@ -446,7 +568,7 @@ class CloneStorageCreator(_StorageCreator):
             clone_block_size = 1024 * 1024 * 10
             sparse = False
 
-        logging.debug("Local Cloning %s to %s, sparse=%s, block_size=%s",
+        log.debug("Local Cloning %s to %s, sparse=%s, block_size=%s",
                       self._input_path, self._output_path,
                       sparse, clone_block_size)
 
@@ -491,7 +613,7 @@ class ManagedStorageCreator(_StorageCreator):
     """
     Handles storage creation via libvirt APIs. All the actual creation
     logic lives in StorageVolume, this is mostly about pulling out bits
-    from that class and mapping them to VirtualDisk elements
+    from that class and mapping them to DeviceDisk elements
     """
     def __init__(self, conn, vol_install):
         _StorageCreator.__init__(self, conn)
